#include<stdio.h>
#include<stdlib.h>
#include<math.h>

/*************************************************************************/
/*     this is the program for calculating the difference between        */
/* the wire which the beam hit at and the position which is predicted by */
/* the minimum square method from three points.                          */
/*************************************************************************/

double difference_x( float channel_x1, float channel_x2, float channel_x3, float channel_x4, int MWPC_number )
{
#define n 3      /* the number of MWPC */

  /*#########################################################*/
  /*#   Set the position of each MWPC.  { dummy , x, y, z } #*/
  /*#########################################################*/
  double MWPC_1x[4] = {0, 1.0, 1663.0, 2314.5};      
  double MWPC_2x[4] = {0,-1.2, 1663.0, 6589.0};    
  double MWPC_3x[4] = {0, 1.0, 1663.0, 9166.0};  
  double MWPC_4x[4] = {0, 1.2, 1663.0, 8979.0};  
  double position_1x,position_2x,position_3x,position_4x;
  double sum_zx,sum_z,sum_x,sum_zz;
  double grad_x,dx;
  double distance,POSITION;
  double x_position,difference_x;
  /*--------------------------------------------------------------------*/
  /* the first component of each array will not be used in this program */
  /*--------------------------------------------------------------------*/

  /*#############################################*/
  /*#    transformation. (channel) ---> (mm)    #*/
  /*#############################################*/
  position_1x = ( channel_x1 )*2 - ( 24.5 * 2 ) + MWPC_1x[1];
  position_2x = ( channel_x2 )*2 - ( 24.5 * 2 ) + MWPC_2x[1];
  position_3x = ( channel_x3 )*2 - ( 24.5 * 2 ) + MWPC_3x[1];
  position_4x = ( channel_x4 )*2 - ( 32.5 * 2 ) + MWPC_4x[1];
  /*---------------------------------------------------------------------*/
  /* the number of wire is 48(MWPC1,2,3) and 64(MWPC4)                   */
  /*---------------------------------------------------------------------*/

  /*###########################################*/
  /*#  Set the distance(mm) and POSITION(mm). #*/
  /*###########################################*/
  if( MWPC_number == 1 )
    {
      distance = MWPC_1x[3];
      POSITION = ( channel_x1 )*2 - ( 24.5 * 2 ) + MWPC_1x[1];
      sum_zx =  (MWPC_2x[3]*position_2x)+(MWPC_3x[3]*position_3x)+(MWPC_4x[3]*position_4x);
      sum_z  =  ( MWPC_2x[3] + MWPC_3x[3] + MWPC_4x[3] );
      sum_x  =  ( position_2x + position_3x + position_4x );
      sum_zz =  (MWPC_2x[3]*MWPC_2x[3])+(MWPC_3x[3]*MWPC_3x[3])+(MWPC_4x[3]*MWPC_4x[3]);
    }
  if( MWPC_number == 2 )
    {
      distance = MWPC_2x[3];
      POSITION = ( channel_x2 )*2 - ( 24.5 * 2 ) + MWPC_2x[1];
      sum_zx =  (MWPC_1x[3]*position_1x)+(MWPC_3x[3]*position_3x)+(MWPC_4x[3]*position_4x);
      sum_z  =  ( MWPC_1x[3] + MWPC_3x[3] + MWPC_4x[3] );
      sum_x  =  ( position_1x + position_3x + position_4x );
      sum_zz =  (MWPC_1x[3]*MWPC_1x[3])+(MWPC_3x[3]*MWPC_3x[3])+(MWPC_4x[3]*MWPC_4x[3]);
    }
  if( MWPC_number == 3 )
    {
      distance = MWPC_3x[3];
      POSITION = ( channel_x3 )*2 - ( 24.5 * 2 ) + MWPC_3x[1];
      sum_zx =  (MWPC_1x[3]*position_1x)+(MWPC_2x[3]*position_2x)+(MWPC_4x[3]*position_4x);
      sum_z  =  ( MWPC_1x[3] + MWPC_2x[3] + MWPC_4x[3] );
      sum_x  =  ( position_1x + position_2x + position_4x );
      sum_zz =  (MWPC_1x[3]*MWPC_1x[3])+(MWPC_2x[3]*MWPC_2x[3])+(MWPC_4x[3]*MWPC_4x[3]);
    }
  if( MWPC_number == 4 )
    {
      distance = MWPC_4x[3];
      POSITION = ( channel_x4 )*2 - ( 32.5 * 2 ) + MWPC_4x[1];
      sum_zx =  (MWPC_1x[3]*position_1x)+(MWPC_2x[3]*position_2x)+(MWPC_3x[3]*position_3x);
      sum_z  =  ( MWPC_1x[3] + MWPC_2x[3] + MWPC_3x[3] );
      sum_x  =  ( position_1x + position_2x + position_3x );
      sum_zz =  (MWPC_1x[3]*MWPC_1x[3])+(MWPC_2x[3]*MWPC_2x[3])+(MWPC_3x[3]*MWPC_3x[3]);
    }
  
  /*#############################################*/
  /*     define the path of the beam particle    */
  /*  by using the minimum square method.        */
  /*#############################################*/
  grad_x =  ( ( n*sum_zx ) - ( sum_z*sum_x ) )/( ( n*sum_zz ) - ( sum_z*sum_z ) );
  dx     =  ( (sum_zz*sum_x) - (sum_z*sum_zx) )/( ( n*sum_zz ) - ( sum_z*sum_z ) );
  x_position = (grad_x)*(distance) + dx;
  difference_x = x_position - POSITION;
  return difference_x;
}


double difference_y( float channel_y1, float channel_y2, float channel_y3, float channel_y4, int MWPC_number )
{
#define n 3
  double MWPC_1y[4] = {0, 0.0, 1663.0, 2332.5};     
  double MWPC_2y[4] = {0, 0.0, 1662.5, 6571.0};    
  double MWPC_3y[4] = {0, 0.0, 1662.5, 9184.0};  
  double MWPC_4y[4] = {0, 0.0, 1663.5, 8969.0};  
  double position_1y,position_2y,position_3y,position_4y;
  double sum_zy,sum_z,sum_y,sum_zz;
  double distance,POSITION;
  double grad_y,dy;
  double y_position,difference_y;

  position_1y = -( channel_y1 )*2 + ( 24.5 * 2 ) + MWPC_1y[2];
  position_2y = -( channel_y2 )*2 + ( 24.5 * 2 ) + MWPC_2y[2];
  position_3y = -( channel_y3 )*2 + ( 24.5 * 2 ) + MWPC_3y[2];
  position_4y = -( channel_y4 )*2 + ( 32.5 * 2 ) + MWPC_4y[2];

  if( MWPC_number == 1 )
    {
      distance = MWPC_1y[3];
      POSITION = -( channel_y1 )*2 + ( 24.5 * 2 ) + MWPC_1y[2];
      sum_zy =  (MWPC_2y[3]*position_2y)+(MWPC_3y[3]*position_3y)+(MWPC_4y[3]*position_4y);
      sum_z  =  ( MWPC_2y[3] + MWPC_3y[3] + MWPC_4y[3] );
      sum_y  =  ( position_2y + position_3y + position_4y );
      sum_zz =  (MWPC_2y[3]*MWPC_2y[3])+(MWPC_3y[3]*MWPC_3y[3])+(MWPC_4y[3]*MWPC_4y[3]);
    }
  if( MWPC_number == 2 )
    {
      distance = MWPC_2y[3];
      POSITION = -( channel_y2 )*2 + ( 24.5 * 2 ) + MWPC_2y[2];
      sum_zy =  (MWPC_1y[3]*position_1y)+(MWPC_3y[3]*position_3y)+(MWPC_4y[3]*position_4y);
      sum_z  =  ( MWPC_1y[3] + MWPC_3y[3] + MWPC_4y[3] );
      sum_y  =  ( position_1y + position_3y + position_4y );
      sum_zz =  (MWPC_1y[3]*MWPC_1y[3])+(MWPC_3y[3]*MWPC_3y[3])+(MWPC_4y[3]*MWPC_4y[3]);
    }
  if( MWPC_number == 3 )
    {
      distance = MWPC_3y[3];
      POSITION = -( channel_y3 )*2 + ( 24.5 * 2 ) + MWPC_3y[2];
      sum_zy =  (MWPC_1y[3]*position_1y)+(MWPC_2y[3]*position_2y)+(MWPC_4y[3]*position_4y);
      sum_z  =  ( MWPC_1y[3] + MWPC_2y[3] + MWPC_4y[3] );
      sum_y  =  ( position_1y + position_2y + position_4y );
      sum_zz =  (MWPC_1y[3]*MWPC_1y[3])+(MWPC_2y[3]*MWPC_2y[3])+(MWPC_4y[3]*MWPC_4y[3]);
    }
  if( MWPC_number == 4 )
    {
      distance = MWPC_4y[3];
      POSITION = -( channel_y4 )*2 + ( 32.5 * 2 ) + MWPC_4y[2];
      sum_zy =  (MWPC_1y[3]*position_1y)+(MWPC_2y[3]*position_2y)+(MWPC_3y[3]*position_3y);
      sum_z  =  ( MWPC_1y[3] + MWPC_2y[3] + MWPC_3y[3] );
      sum_y  =  ( position_1y + position_2y + position_3y );
      sum_zz =  (MWPC_1y[3]*MWPC_1y[3])+(MWPC_2y[3]*MWPC_2y[3])+(MWPC_3y[3]*MWPC_3y[3]);
    }
  
  grad_y = ( (n*sum_zy) - (sum_z*sum_y) )/( (n*sum_zz) - (sum_z*sum_z) );
  dy = ( (sum_zz*sum_y) - (sum_z*sum_zy) )/( (n*sum_zz) - (sum_z*sum_z) );
  y_position = (grad_y)*(distance) + dy;
  difference_y = y_position - POSITION;
  return difference_y;
}
